package com.miya.huihua.microservicebase.mq;

import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.Producer;
import com.aliyun.openservices.ons.api.SendCallback;
import com.aliyun.openservices.ons.api.SendResult;
import com.aliyun.openservices.shade.com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

import static com.miya.huihua.microservicebase.common.Constants.TAG_ALL;

/**
 * @author zhouxu
 * @createTime 2020/01/17
 */

@Service
@Slf4j
public class AliProducerImpl implements AliProducer {

    @Autowired(required = false)
    private OnsManager onsManager;


    @Override
    public void putEvent(String topic, Object o, String... tags) {
        Producer producer = getProducer(topic);
        Message message = new Message(topic, tags.length < 1 ? TAG_ALL : Arrays.asList(tags).stream().collect(Collectors.joining("||")), JSON.toJSONString(o).getBytes());
        try {
            SendResult sendResult = producer.send(message);
            if (sendResult != null) {
                log.info("send msg success,Topic is :{},getMessageId is :{}", sendResult.getTopic(), sendResult.getMessageId());
            }
        } catch (Exception e) {
            log.error("Send mq message failed. Topic is:{}, Reson is : {}", topic, e.getMessage());
            e.printStackTrace();
        }
    }

    @Override
    public void putAsyncEvent(String topic, Object o, SendCallback sendCallback, String... tags) {
        Producer producer = getProducer(topic);
        Message message = new Message(topic, tags.length < 1 ? TAG_ALL : Arrays.asList(tags).stream().collect(Collectors.joining("||")), JSON.toJSONString(o).getBytes());
        producer.sendAsync(message, sendCallback);
    }

    private Producer getProducer(String topic) {
        ConcurrentHashMap<String, Producer> concurrentHashMap = onsManager.getProducerConcurrentHashMap();
        return concurrentHashMap.get(topic);
    }
}
