package com.miya.huihua.microservicebase.mq;

import com.aliyun.openservices.ons.api.Consumer;
import com.aliyun.openservices.ons.api.ONSFactory;
import com.aliyun.openservices.ons.api.Producer;
import com.aliyun.openservices.ons.api.PropertyKeyConst;
import com.miya.huihua.microservicebase.data.config.MultiOnsProperties;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.SmartLifecycle;
import org.springframework.stereotype.Component;

import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author zhouxu
 * @createTime 2020/01/17
 */

@Component
@Slf4j
public class OnsManager implements SmartLifecycle {

    @Autowired(required = false)
    private MultiOnsProperties multiOnsProperties;

    public volatile boolean isRunning = false;

    /**
     * 生产者
     */
    private ConcurrentHashMap<String, Producer> producerConcurrentHashMap = new ConcurrentHashMap<>();

    /**
     * 消费者
     */
    private ConcurrentHashMap<String, Consumer> consumerConcurrentHashMap = new ConcurrentHashMap<>();

    @Override
    public boolean isAutoStartup() {
        return true;
    }

    @Override
    public void stop(Runnable runnable) {
        runnable.run();
        isRunning = false;
    }

    @Override
    public void start() {
        if (multiOnsProperties == null || multiOnsProperties.getOnsProperties() == null || multiOnsProperties.getOnsProperties().isEmpty()) {
            log.debug("未获取到正确的ONS配置...");
            isRunning = false;
            return;
        }
        multiOnsProperties.getOnsProperties().stream().forEach(e -> {
            Properties properties = new Properties();
            properties.setProperty(PropertyKeyConst.ONSAddr, multiOnsProperties.getOnsAddr());
            properties.setProperty(PropertyKeyConst.AccessKey, multiOnsProperties.getAccessKey());
            properties.setProperty(PropertyKeyConst.SecretKey, multiOnsProperties.getSecretKey());
            properties.setProperty(PropertyKeyConst.SendMsgTimeoutMillis, String.valueOf(multiOnsProperties.getTimeoutMillis()));
            // 您在控制台创建的 Group ID
            if (e.getProducerEnable()) {
                properties.setProperty(PropertyKeyConst.GROUP_ID, e.getGroupId());
                Producer producer = ONSFactory.createProducer(properties);
                producer.start();
                producerConcurrentHashMap.putIfAbsent(e.getTopic(), producer);
            }
            if (e.getConsumerEnable()) {
                Consumer consumer = ONSFactory.createConsumer(properties);
                consumerConcurrentHashMap.putIfAbsent(e.getTopic(), consumer);
            }
        });
        isRunning = true;
    }

    @Override
    public void stop() {
        producerConcurrentHashMap.forEach((k, v) -> v.shutdown());
    }

    @Override
    public boolean isRunning() {
        return isRunning;
    }

    @Override
    public int getPhase() {
        return 0;
    }


    public ConcurrentHashMap<String, Producer> getProducerConcurrentHashMap() {
        return producerConcurrentHashMap;
    }

    public ConcurrentHashMap<String, Consumer> getConsumerConcurrentHashMap() {
        return consumerConcurrentHashMap;
    }
}
